package com.eldos.sp;

import SecureBlackbox.Base.SBStrUtils;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.Base.TElX509Certificate;
import SecureBlackbox.SAMLCommon.*;
import SecureBlackbox.XML.TElXMLDOMDocument;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.InputStream;

public class ACSServlet extends HttpServlet {

    private static final long MAX_ISSUE_INSTANT_TIME_DIFF = 60000;

    private String idpSignCert, spEncryptionCert;

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        idpSignCert = config.getInitParameter("idp_sign_cert");
        spEncryptionCert = config.getInitParameter("sp_encryption_cert");
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        TElSAMLPOSTBinding binding = new TElSAMLPOSTBinding();
        binding.setMode(TSBSAMLPOSTBindingMode.spmServer);

        String body = streamToStr(req.getInputStream());
        String saml = binding.loadResponse(body);
        String state = binding.getRelayState();

        TElXMLDOMDocument doc = new TElXMLDOMDocument();
        byte[] samlBytes = SBStrUtils.strToUTF8(saml);
        TElMemoryStream ms = new TElMemoryStream(samlBytes, 0, samlBytes.length);
        doc.loadFromStream(ms);

        if ( ! "Response".equals(doc.getDocumentElement().getLocalName())) {
            resp.setStatus(400); // Bad request
            return;
        }

        TElX509Certificate idpCert = new TElX509Certificate();
        idpCert.loadFromFileAuto(idpSignCert, "");

        TElSAMLSignatureHandler sigHandler = new TElSAMLSignatureHandler();
        sigHandler.setCertificate(idpCert);
        sigHandler.setValidateReferences(true);

        TElSAMLResponseElement samlResp = new TElSAMLResponseElement();
        samlResp.setHandler(sigHandler);
        samlResp.loadFromXML(doc.getDocumentElement());

        // check response
        if ( ! SBXMLSAMLCore.SAMLVersion.equals(samlResp.getVersion())) {
            resp.setStatus(400); // Bad request
            return;
        }

        long diff = SBUtils.dateTimeToUnixTime(samlResp.getIssueInstant()) - SBUtils.dateTimeToUnixTime(SBUtils.utcNow());
        if (diff > MAX_ISSUE_INSTANT_TIME_DIFF) {
            resp.setStatus(400); // Bad request
            return;
        }

        HttpSession session = req.getSession(false);
        if (session == null) {
            resp.setStatus(400); // Bad request
            return;
        }

        String reqID = (String) session.getAttribute("samlReqID");
        if (reqID != null && samlResp.getInResponseTo() != null && ( ! reqID.equals(samlResp.getInResponseTo()))) {
            resp.setStatus(400); // Bad request
            return;
        }

        if ( ! SBXMLSAMLProtocol.xmlSAMLStatusSuccess.equals(samlResp.getStatus().getStatusCode().getValue())) {
            resp.setStatus(400); // Bad request
            return;
        }

        // check assertions
        TElSAMLAssertionType at;
        TElSAMLAssertionElement assertion;

        for (int i = 0; i < samlResp.getAssertionCount(); i++) {
            at = samlResp.getAssertions(i);

            if (at instanceof TElSAMLEncryptedAssertionElement) {
                // encrypted assertion
                TElSAMLEncryptedAssertionElement ea = (TElSAMLEncryptedAssertionElement) at;

                TElX509Certificate encCert = new TElX509Certificate();
                encCert.loadFromFileAuto(spEncryptionCert, "");

                TElSAMLEncryptionHandler encHandler = new TElSAMLEncryptionHandler();
                encHandler.setCertificate(encCert);

                ea.setHandler(encHandler);
                ea.decrypt();

                if (ea.getDecrypted())
                    assertion = ea;
                else
                    throw new RuntimeException("Decryption error!");
            } else {
                // check
                assertion = (TElSAMLAssertionElement)at;
            }

            // check statements
            TElSAMLAuthnStatementElement statement;
            for (int j = 0; j < assertion.getStatementCount(); j++) {
                if (assertion.getStatements(j) instanceof TElSAMLAuthnStatementElement) {
                    statement = (TElSAMLAuthnStatementElement) assertion.getStatements(j);
                    // todo
                }
            }
        }

        // authorized
        session.setAttribute("auth", true);

        resp.setStatus(302); // Redirect
        resp.setHeader("Location", state);
    }

    private String streamToStr(InputStream is) throws IOException {
        StringBuilder sb = new StringBuilder();
        byte[] buf = new byte[1024];
        int r;

        do {
            if ((r = is.read(buf)) > 0)
                sb.append(new String(buf, 0, r, "utf-8"));
        } while (r > 0);

        return sb.toString();
    }
}
